@@ -1,6 +1,6 @@
 <?php
 
-/**
+/*
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
  | Copyright (C) 2005-2012, The Roundcube Dev Team                       |
@@ -38,13 +38,6 @@ class rcube_db_mysql extends rcube_db
      */
     public function __construct($db_dsnw, $db_dsnr = '', $pconn = false)
     {
-        if (version_compare(PHP_VERSION, '5.3.0', '<')) {
-            rcube::raise_error(array('code' => 600, 'type' => 'db',
-                'line' => __LINE__, 'file' => __FILE__,
-                'message' => "MySQL driver requires PHP >= 5.3, current version is " . PHP_VERSION),
-                true, true);
-        }
-
         parent::__construct($db_dsnw, $db_dsnr, $pconn);
 
         // SQL identifiers quoting
